function [Xs,Rs_ie,Rs_eq] = do_ls_ranking_with_r(A_prior,b_prior,points,comps,opts)
% do least squares rank aggregation
% comps should have a field named "pred_prob", a 1x3 matrix for the
% probabilites of 'E','1' and '2'
% this version also solves for the ratio for equalities and inequalites
%
% solves min_x x'*A_prior*x + soft equality and inequality constraints s.t
% x>lb and x<ub
relation_field = opts.relation_field;
relation_field_score = [opts.relation_field '_score'];

params = opts.ls_ranking_params;
lb = params.lower_bound;
ub = params.upper_bound;

%% get equality and inequality pairs
if params.use_all_preds
    ie_pairs = zeros(length(comps)*2,2);
    eq_pairs = zeros(length(comps),2);
    eq_ws = zeros(length(comps),1);
    ie_ws = zeros(length(comps)*2,1);
    for t=1:length(comps)
        c = comps(t); 
        p1 = points(c.point1);
        p2 = points(c.point2);

        if ~p1.opaque || ~p2.opaque
            continue
        end

        eq_pairs(t,:) = [p1.id, p2.id];
        eq_ws(t) = c.pred_prob(1);

        ie_pairs(t,:) = [p2.id, p1.id];
        ie_ws(t) = c.pred_prob(2);

        ie_pairs(length(comps)+t,:) = [p1.id, p2.id];
        ie_ws(length(comps)+t) = c.pred_prob(3);
    end
else
    ie_pairs = []; eq_pairs = [];
    ie_ws = []; eq_ws = [];
    for t=1:length(comps)
        c = comps(t); 
        p1 = points(c.point1);
        p2 = points(c.point2);

        if c.(relation_field_score)>0
            if c.(relation_field)=='E'
                eq_pairs = [eq_pairs; p1.id, p2.id];
                eq_ws = [eq_ws; c.(relation_field_score)];
            elseif c.(relation_field)=='1'
                ie_pairs = [ie_pairs; p2.id, p1.id];
                ie_ws = [ie_ws; c.(relation_field_score)];
            else
                ie_pairs = [ie_pairs; p1.id, p2.id];
                ie_ws = [ie_ws; c.(relation_field_score)];
            end
        end
    end
end

if isfield(params,'score_threshold')
%         pairs_to_delete = find(ie_ws<params.score_threshold);
%         ie_pairs(pairs_to_delete,:) = [];
%         ie_ws(pairs_to_delete,:) = [];
% 
%         pairs_to_delete = find(eq_ws<params.score_threshold);
%         eq_pairs(pairs_to_delete,:) = [];
%         eq_ws(pairs_to_delete,:) = [];
    ie_ws(ie_ws<params.score_threshold) = 0;
    eq_ws(eq_ws<params.score_threshold) = 0;
end

if params.use_uniform_weights
    if params.use_all_preds
        warning('If using all predictions then using uniform weights is a bad idea.');
    end
    ie_ws = ones(size(ie_ws));
    eq_ws = ones(size(eq_ws));
end

%% build and solve quadratic system
npoints = length(points);

ie_pairs = double(ie_pairs);
eq_pairs = double(eq_pairs);

num_eq = size(eq_pairs,1);
num_ie = size(ie_pairs,1);

% build ineqaulity matrix
ie_inds = 1:num_ie;
Aie = sparse([ie_inds(:); ie_inds(:); ie_inds(:)],[ie_pairs(:,1); ie_pairs(:,2); npoints+ie_inds(:)],[-ie_ws.*ones(size(ie_pairs,1),1); ie_ws.*ones(size(ie_pairs,1),1); -ie_ws.*ones(num_ie,1)],size(ie_pairs,1),npoints+num_eq+num_ie);
bie = 0*params.ie_ratio.*ones(size(ie_pairs,1),1);

% build equality matrix
eq_inds = 1:size(eq_pairs,1);
Aeq = sparse([eq_inds(:); eq_inds(:); eq_inds(:)],[eq_pairs(:,1); eq_pairs(:,2); npoints+num_ie+eq_inds(:)],[eq_ws.*ones(size(eq_pairs,1),1); -eq_ws.*ones(size(eq_pairs,1),1); -eq_ws.*ones(num_eq,1)],size(eq_pairs,1),npoints+num_eq+num_ie);
beq = zeros(size(eq_pairs,1),1);

%% add prior for ratio
AR = sparse([npoints+ie_inds(:);npoints+num_ie+eq_inds(:)],[npoints+ie_inds(:);npoints+num_ie+eq_inds(:)],[0.5*ones(num_ie,1)/params.ratio_solver_ie_var;0.5*ones(num_eq,1)/params.ratio_solver_eq_var],npoints+num_ie+num_eq, npoints+num_ie+num_eq);
bR = [zeros(npoints,1);params.ie_ratio*ones(num_ie,1)/params.ratio_solver_ie_var;zeros(num_eq,1)];

% add zeros for the prior
A_prior(npoints+num_ie+num_eq,npoints+num_ie+num_eq) = 0;
b_prior = [b_prior; zeros(num_ie+num_eq,1)];

% solve system
A = (params.eq_weight*(Aeq'*Aeq) + params.ie_weight*(Aie'*Aie) + A_prior + AR);
b = (params.eq_weight*(Aeq'*beq) + params.ie_weight*(Aie'*bie) + b_prior + bR);
XRs = quadprog(A,-b,[],[],[],[],[lb.*ones(npoints,1);-Inf*ones(num_ie,1);-Inf*ones(num_eq,1)],[ub.*ones(npoints,1);0*ones(num_ie,1);Inf*ones(num_eq,1)]);

Xs = XRs(1:npoints);
Rs_ie = XRs(npoints+(1:num_ie));
Rs_eq = XRs(npoints+num_ie+(1:num_eq));